@@ -16,10 +16,17 @@ const Volume = imports.ui.status.volume;
 const { Menu } = Lib.menu.menu;
 const { PercentageLabel } = Lib.widget.percentageLabel;
 
+const VolumeType = {
+    OUTPUT: 0,
+    INPUT: 1,
+};
+
 
 /**
  * Custom indicator with dropdown menu.
  * Copied from status/volume.js
+ *
+ * @copypaste: Needs code to not initialize volume menu and all its events.
  */
 var Indicator = GObject.registerClass(class Indicator extends PanelMenu.SystemIndicator
 {
@@ -33,18 +40,19 @@ var Indicator = GObject.registerClass(class Indicator extends PanelMenu.SystemIn
 
         this._primaryIndicator = this._addIndicator();
         this._inputIndicator = this._addIndicator();
-        this._control = mixer.control;
 
+        this._primaryIndicator.reactive = true;
+        this._inputIndicator.reactive = true;
+
+        this._primaryIndicator.connect('scroll-event',
+            (actor, event) => Volume.Indicator.prototype._handleScrollEvent.apply(this, [VolumeType.OUTPUT, event]));
+        this._inputIndicator.connect('scroll-event',
+            (actor, event) => Volume.Indicator.prototype._handleScrollEvent.apply(this, [VolumeType.INPUT, event]));
+
+        this._control = mixer.control;
         this._volumeMenu = new Menu(mixer, options);
-        this._volumeMenu.actor.add_style_class_name('svm-integrated-menu');
         this._volumeMenu.connect('output-icon-changed', this.updateOutputIcon.bind(this));
 
-        if (options.showPercentageLabel) {
-            this._percentageLabel = new PercentageLabel(mixer);
-            this.add(this._percentageLabel);
-        }
-
-        // copy paste code here
         this._inputIndicator.visible = this._volumeMenu.getInputVisible();
         this._volumeMenu.connect('input-visible-changed', () => {
             this._inputIndicator.visible = this._volumeMenu.getInputVisible();
@@ -57,27 +65,22 @@ var Indicator = GObject.registerClass(class Indicator extends PanelMenu.SystemIn
         });
 
         this.menu.addMenuItem(this._volumeMenu);
-    }
 
-    /**
-     * We can mimic the original Indicator's handling of scroll events.
-     */
-    vfunc_scroll_event() {
-        Volume.Indicator.prototype.vfunc_scroll_event.apply(this, arguments);
+        this._volumeMenu.actor.add_style_class_name('svm-integrated-menu');
 
-        // tell all upstream consumers we handled this event
-        return Clutter.EVENT_STOP;
+        if (options.showPercentageLabel) {
+            this._percentageLabel = new PercentageLabel(mixer);
+            this.add(this._percentageLabel);
+        }
     }
 
     updateOutputIcon() {
         let icon = this._volumeMenu.getOutputIcon();
 
         if (icon) {
-            this.show();
             this._primaryIndicator.icon_name = icon;
             this._primaryIndicator.visible = true;
         } else {
-            this.hide();
             this._primaryIndicator.visible = false;
         }
     }
